@@ -21,29 +21,18 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #----------------------------------------------------------------------------
-
-# module folder name
 import subprocess
-
-modulesFolder = 'modules'
 import sys
-# add module folder to path
-sys.path.append(modulesFolder)
 import time
-startTimestamp = time.time()
-import math
-# set current directory to the directory
-# of this file
-# like this, modRana can be run from an absolute path
-# eq.: ./opt/modrana/modrana.py -u QML -d n9
-import os
-os.chdir(os.path.dirname(os.path.abspath(__file__)))
 
+startTimestamp = time.time()
+import os
 import marshal
 import traceback
-from math import radians
+import imp
 # import core modules/classes
 from core import startup
+from core import utils
 from core import paths
 from core import configs
 from core import gs
@@ -51,37 +40,24 @@ from core import gs
 # record that imports-done timestamp
 importsDoneTimestamp = time.time()
 
-def createFolderPath(newPath):
-  """
-  Create a path for a directory and all needed parent folders
-  -> parent directories will be created
-  -> if directory already exists, then do nothing
-  -> if there is another filesystem object (like a file) with the same name, raise an exception
-  """
-  if not newPath:
-    print("cannot create folder, wrong path: ", newPath)
-    return False
-  if os.path.isdir(newPath):
-    return True
-  elif os.path.isfile(newPath):
-    print("cannot create directory, file already exists: '%s'" % newPath)
-    return False
-  else:
-    print("creating path: %s" % newPath)
-    head, tail = os.path.split(newPath)
-    if head and not os.path.isdir(head):
-        os.mkdirs(head)
-    if tail:
-        os.mkdir(newPath)
-    return True
+MAIN_MODULES_FOLDER = 'modules'
+DEVICE_MODULES_FOLDER = os.path.join(MAIN_MODULES_FOLDER, "device_modules")
+GUI_MODULES_FOLDER = os.path.join(MAIN_MODULES_FOLDER, "gui_modules")
+ALL_MODULE_FOLDERS = [
+  MAIN_MODULES_FOLDER,
+  DEVICE_MODULES_FOLDER,
+  GUI_MODULES_FOLDER
+]
 
-class ModRana:
+
+class ModRana(object):
   """
   This is THE main modRana class.
   """
+
   def __init__(self):
     self.timing = []
-    self.addCustomTime("modRana start",startTimestamp)
+    self.addCustomTime("modRana start", startTimestamp)
     self.addCustomTime("imports done", importsDoneTimestamp)
 
     # constants & variable initialization
@@ -95,7 +71,7 @@ class ModRana:
     self.watches = {} # List of data change watches
     self.maxWatchId = 0
 
-    self.initInfo={
+    self.initInfo = {
       'modrana': self,
       'device': None, # TODO: do this directly
       'name': ""
@@ -129,7 +105,7 @@ class ModRana:
     # add the paths handling core module
     self.paths = paths.Paths(self)
 
-    # print version string
+    # print(version string)
     version = self.paths.getVersionString()
     if version is None:
       version = "unknown version"
@@ -139,7 +115,7 @@ class ModRana:
     self.configs = configs.Configs(self)
 
     # load persistent options
-    self.optLoadingOK= self._loadOptions()
+    self.optLoadingOK = self._loadOptions()
 
     # check if upgrade took place
 
@@ -147,7 +123,7 @@ class ModRana:
       savedVersionString = self.get('modRanaVersionString', "")
       versionStringFromFile = self.paths.getVersionString()
       if savedVersionString != versionStringFromFile:
-        print "modRana: possible upgrade detected"
+        print("modRana: possible upgrade detected")
         self._postUpgradeCheck()
 
     # save current version string
@@ -188,6 +164,7 @@ class ModRana:
     else: # no device specified from CLI
       # try to auto-detect the current device
       from core import platform_detection
+
       device = platform_detection.getBestDeviceModuleId()
 
     device = device.lower() # convert to lowercase
@@ -196,19 +173,26 @@ class ModRana:
 
     # get GUI ID from the CLI argument
     if self.args.u:
-      self.GUIString = self.args.u
+      try:
+        self.GUIString = self.args.u.split(":")[0]
+      except Exception, e:
+        print('modRana: splitting the GUI string failed')
+        print(e)
     else: # no ID specified
       # the N900 device module needs the GUIString
       # at startup
       if device == "n900":
         self.GUIString = "GTK"
+
+    # set the pre-import-visible GUIString
+    # for the device module
     gs.GUIString = self.GUIString
 
     ## load the device specific module
 
     # NOTE: other modules need the device and GUI modules
     # during init
-    deviceModulesPath = os.path.join(modulesFolder, "device_modules")
+    deviceModulesPath = os.path.join(MAIN_MODULES_FOLDER, "device_modules")
     sys.path.append(deviceModulesPath)
     dmod = self._loadModule("device_%s" % device, "device")
     if dmod is None:
@@ -227,25 +211,38 @@ class ModRana:
         self.GUIString = ids[0]
       else:
         self.GUIString = "GTK" # fallback
-      # export the GUI string
-      gs.GUIString = self.GUIString
+        # export the GUI string
+      # set the pre-import visible GUI string and subtype
+    splitGUIString = self.GUIString.split(":")
+    gs.GUIString = splitGUIString[0]
+    if len(splitGUIString) >= 2:
+      gs.GUISubtypeString = splitGUIString[1]
 
-    # TODO: if loading GUI module fails, retry other modules in
-    # order of preference as provided by the  device module
+      # TODO: if loading GUI module fails, retry other modules in
+      # order of preference as provided by the  device module
 
   def _loadGUIModule(self):
     """load the GUI module"""
 
     # add the GUI module folder to path
-    GUIModulesPath = os.path.join(modulesFolder, "gui_modules")
+    GUIModulesPath = os.path.join(MAIN_MODULES_FOLDER, "gui_modules")
     sys.path.append(GUIModulesPath)
+    gui = None
+    splitGUIString = self.GUIString.split(":")
 
-    if self.GUIString == "GTK":
+    GUIModuleId = splitGUIString[0]
+    if len(splitGUIString) > 1:
+      subtypeId = splitGUIString[1]
+    else:
+      subtypeId = None
+
+    if GUIModuleId == "GTK":
       gui = self._loadModule("gui_gtk", "gui")
-    elif self.GUIString == "QML":
+    elif GUIModuleId == "QML":
       gui = self._loadModule("gui_qml", "gui")
       # make device module available to the GUI module
     if gui:
+      gui.setSubtypeId(subtypeId)
       gui.dmod = self.dmod
     self.gui = gui
 
@@ -259,12 +256,12 @@ class ModRana:
     loadModule = self._loadModule
 
     # get possible module names
-    moduleNames = self._getModuleNamesFromFolder(modulesFolder)
+    moduleNames = self._getModuleNamesFromFolder(MAIN_MODULES_FOLDER)
     # load if possible
     for moduleName in moduleNames:
-        # filter out .py
-        moduleName = moduleName.split('.')[0]
-        loadModule(moduleName, moduleName[4:])
+      # filter out .py
+      moduleName = moduleName.split('.')[0]
+      loadModule(moduleName, moduleName[4:])
 
     print("Loaded all modules in %1.2f ms, initialising" % (1000 * (time.clock() - start)))
     self.addTime("all modules loaded")
@@ -289,26 +286,56 @@ class ModRana:
     # add last timing checkpoint
     self.addTime("all modules initialized")
 
-  def _getModuleNamesFromFolder(self,folder):
-    """list a given folder and find all possible module names"""
-    return filter(lambda x: x[0:4]=="mod_" and x[-4:]!=".pyc",os.listdir(folder))
+  def _getModuleNamesFromFolder(self, folder, prefix='mod_'):
+    """list a given folder and find all possible module names
+    Module names:
+    Module names start with the "mod_" and don't end with .pyc or .pyo.
+    Consequences:
+    Valid modules need to have an existing .py file or be folder-modules
+    (don't name a folder module mod_foo.pyc :) ), even if they are
+    actually loaded from the .pyc or .pyo in the end.
+    This is done so that dangling .pyc/.pyo file from a module
+    that was removed are not loaded by mistake.
+    This situation shouldn't really happen if modRana is installed from a package,
+    as all .pyc files are purged during package upgrade and regenerated."""
+    return filter(
+      lambda x:
+      x[0:len(prefix)] == prefix
+      and
+      (x[-4:] not in ('.pyc', '.pyo')), os.listdir(folder)
+    )
+
+  def _listAvailableDeviceModulesByID(self):
+    moduleNames = self._getModuleNamesFromFolder(DEVICE_MODULES_FOLDER, prefix='device_')
+    # remove the device_ prefix and return the results
+    # NOTE: .pyc & .pyo should be removed already in _getModuleNamesFromFolder()
+    # laos sort the module names alphabetically
+    return sorted(map(lambda x: x[7:].replace('.py', ''), moduleNames))
+
+  def _listAvailableGUIModulesByID(self):
+    return self._getModuleNamesFromFolder(GUI_MODULES_FOLDER)
 
   def _loadModule(self, importName, modRanaName):
     """load a single module by name from path"""
     startM = time.clock()
+    fp = None
     try:
-      a = __import__(importName)
+      fp, pathName, description = imp.find_module(importName, ALL_MODULE_FOLDERS)
+      a = imp.load_module(importName, fp, pathName, description)
       initInfo = self.initInfo
-      name = modRanaName
-      initInfo['name'] = name
+      initInfo['name'] = modRanaName
       module = a.getModule(self.m, self.d, initInfo)
-      self.m[name] = module
-      print( " * %s: %s (%1.2f ms)" % (name, self.m[name].__doc__, (1000 * (time.clock() - startM))) )
+      self.m[modRanaName] = module
+      print( " * %s: %s (%1.2f ms)" % (modRanaName, self.m[modRanaName].__doc__, (1000 * (time.clock() - startM))) )
       return module
     except Exception, e:
       print( "modRana: module: %s/%s failed to load" % (importName, modRanaName) )
+      print(e)
       traceback.print_exc(file=sys.stdout) # find what went wrong
       return None
+    finally:
+      if fp:
+        fp.close()
 
   def _modulesLoadedPreFirstTime(self):
     """this is run after all the modules have been loaded,
@@ -327,7 +354,7 @@ class ModRana:
   def _modulesLoadedPostFirstTime(self):
     """this is run after all the modules have been loaded,
     after before their first time is called"""
-    
+
     # check if redrawing time should be printed to terminal
     if 'showRedrawTime' in self.d and self.d['showRedrawTime'] == True:
       self.showRedrawTime = True
@@ -382,7 +409,7 @@ class ModRana:
     self._saveOptions()
     time.sleep(2) # leave some times for threads to shut down
     print("Shutdown complete")
-    
+
   ## OPTIONS SETTING AND WATCHING ##
 
   def get(self, name, default=None, mode=None):
@@ -395,9 +422,9 @@ class ModRana:
         mode = self.d.get('mode', 'car')
       if mode in self.keyModifiers[name]['modes'].keys():
         # get the dictionary with per mode values
-        multiDict = self.d.get('%s#multi' % name , {})
+        multiDict = self.d.get('%s#multi' % name, {})
         # return the value for current mode
-        return multiDict.get(mode,default)
+        return multiDict.get(mode, default)
       else:
         return self.d.get(name, default)
 
@@ -415,22 +442,22 @@ class ModRana:
       # get the current mode
       if mode is None:
         mode = self.d.get('mode', 'car')
-      # check if there is a modifier for the current mode
+        # check if there is a modifier for the current mode
       if mode in self.keyModifiers[name]['modes'].keys():
         # save it to the name + #multi key under the mode key
         try:
           self.d['%s#multi' % name][mode] = value
         except KeyError: # key not yet created
-          self.d['%s#multi' % name] = {mode : value}
+          self.d['%s#multi' % name] = {mode: value}
       else: # just save to the key as usual
         self.d[name] = value
     else: # just save to the key as usual
       self.d[name] = value
 
     self._notifyWatcher(name, oldValue)
-    """options are normally saved on shutdown,
-    but for some data we want to make sure they are stored and not
-    los for example because of power outage/empty battery, etc."""
+    # options are normally saved on shutdown,
+    # but for some data we want to make sure they are stored and not
+    # lost for example because of power outage/empty battery, etc.
     if save:
       options = self.m.get('options')
       if options:
@@ -449,8 +476,8 @@ class ModRana:
       # purge any key modifiers
       if key in self.keyModifiers.keys():
         del self.keyModifiers[key]
-        """also remove the possibly present
-        alternative states for different modes"""
+        # also remove the possibly present
+        # alternative states for different modes"""
         multiKey = "%s#multi" % key
         if multiKey in self.d:
           del self.d[multiKey]
@@ -459,27 +486,33 @@ class ModRana:
     else:
       print("modrana: can't purge a not-present key: %s" % key)
 
-  def watch(self, key, callback, *args):
+  def watch(self, key, callback, args=None, runNow=False):
     """add a callback on an options key
     callback will get:
     key, newValue, oldValue, *args
 
     NOTE: watch ids should be >0, so that they evaluate as True
     """
+    if not args: args = []
     nrId = self.maxWatchId + 1
-    id = "%d_%s" % (nrId,key)
+    id = "%d_%s" % (nrId, key)
     self.maxWatchId = nrId # TODO: recycle ids ? (alla PID)
     if key not in self.watches:
       self.watches[key] = [] # create the initial list
-    self.watches[key].append((id,callback,args))
+    self.watches[key].append((id, callback, args))
+    # should we now run the callback one ?
+    # -> this is useful for modules that configure
+    # themselves according to an options value at startup
+    if runNow:
+      currentValue = self.get(key, None)
+      callback(key, currentValue, currentValue, *args)
     return id
 
   def removeWatch(self, id):
     """remove watch specified by the given watch id"""
     (nrId, key) = id.split('_')
-
     if key in self.watches:
-      remove = lambda x:x[0]==id
+      remove = lambda x: x[0] == id
       self.watches[key][:] = [x for x in self.watches[key] if not remove(x)]
     else:
       print("modRana: can't remove watch - key does not exist, watchId:", id)
@@ -494,11 +527,11 @@ class ModRana:
     callbacks = self.watches.get(key, None)
     if callbacks:
       for item in callbacks:
-        (id,callback,args) = item
+        (id, callback, args) = item
         # rather supply the old value than None
         newValue = self.get(key, oldValue)
         if callback:
-          if callback(key,oldValue, newValue, *args) == False:
+          if callback(key, oldValue, newValue, *args) == False:
             # remove watches that return False
             self.removeWatch(id)
         else:
@@ -515,41 +548,37 @@ class ModRana:
       defaultValue = options.getKeyDefault(key, None)
     else:
       defaultValue = None
-    oldValue = self.get(key,defaultValue)
+    oldValue = self.get(key, defaultValue)
     if mode is None:
       mode = self.d.get('mode', 'car')
     if key not in self.keyModifiers.keys(): # initialize
-      self.keyModifiers[key] = {'modes':{mode:modifier}}
+      self.keyModifiers[key] = {'modes': {mode: modifier}}
     else:
       self.keyModifiers[key]['modes'][mode] = modifier
 
     # make sure the multi mode dictionary exists
     multiKey = '%s#multi' % key
-    multiDict = self.d.get(multiKey , {})
+    multiDict = self.d.get(multiKey, {})
     self.d[multiKey] = multiDict
 
-    """if the modifier is set for the first time,
-    do we copy the value from the normal key or not ?"""
+    # if the modifier is set for the first time,
+    # do we copy the value from the normal key or not ?
     if copyInitialValue:
       # check if the key is unset for this mode
       if mode not in multiDict:
         # set for first time, copy value        
         self.set(key, self.d.get(key, defaultValue), mode=mode)
-
-    # notify watchers
+        # notify watchers
     self._notifyWatcher(key, oldValue)
 
   def removeKeyModifier(self, key, mode=None):
     """remove key modifier
     NOTE: currently this just makes the key independent
     on the current mode"""
-
-
     # if no mode is provided, use the current one
     if mode is None:
-        mode = self.d.get('mode', 'car')
+      mode = self.d.get('mode', 'car')
     if key in self.keyModifiers.keys():
-      
       # just remove the key modifier preserving the alternative values
       if mode in self.keyModifiers[key]['modes'].keys():
         # get the previous value
@@ -560,14 +589,14 @@ class ModRana:
           defaultValue = options.getKeyDefault(key, None)
         else:
           defaultValue = None
-        oldValue = self.get(key,defaultValue)
+        oldValue = self.get(key, defaultValue)
         del self.keyModifiers[key]['modes'][mode]
         # was this the last key ?
         if len(self.keyModifiers[key]['modes']) == 0:
           # no modes registered - unregister from modifiers
           # TODO: handle non-mode modifiers in the future
           del self.keyModifiers[key]
-        # notify watchers
+          # notify watchers
         self._notifyWatcher(key, oldValue)
         # done
         return True
@@ -593,12 +622,13 @@ class ModRana:
       return False
 
   def getModes(self):
+    """return supported modes"""
     modes = {
-      'cycle':'Cycle',
-      'walk':'Foot',
-      'car':'Car',
-      'train':'Train',
-      'bus':'Bus',
+      'cycle': 'Cycle',
+      'walk': 'Foot',
+      'car': 'Car',
+      'train': 'Train',
+      'bus': 'Bus',
     }
     return modes
 
@@ -614,12 +644,13 @@ class ModRana:
     """handle mode change in regards to key modifiers and option key watchers"""
     # get keys that have both a key modifier and a watcher
     keys = filter(lambda x: x in self.keyModifiers.keys(), self.watches.keys())
-    """ filter out only those keys that have a modifier for the new mode or
-    had a modifier in the previous mode
-    otherwise their value would not change and thus triggering a watch is not necessary """
+    # filter out only those keys that have a modifier for the new mode
+    # or had a modifier in the previous mode
+    # otherwise their value would not change and thus
+    # triggering a watch is not necessary
     keys = filter(
-                  lambda x: newMode in self.keyModifiers[x]['modes'].keys() or oldMode in self.keyModifiers[x]['modes'].keys(),
-                  keys )
+      lambda x: newMode in self.keyModifiers[x]['modes'].keys() or oldMode in self.keyModifiers[x]['modes'].keys(),
+      keys)
     for key in keys:
       # try to get some value if the old value is not available
       options = self.m.get('options', None)
@@ -642,10 +673,13 @@ class ModRana:
     try:
       return dict((k, v) for k, v in inputDict.iteritems() if k[0] != '#')
     except Exception, e:
-      print('options: error while filtering options\nsome nonpersistent keys might have been left in\nNOTE: keys should be strings of length>=1\n', e)
+      print(
+        'options: error while filtering options\nsome nonpersistent keys might have been left in\nNOTE: keys should be strings of length>=1\n',
+        e)
       return self.d
 
   def _saveOptions(self):
+    """save the persistent dictionary to file"""
     print("modRana: saving options")
     try:
       f = open(self.paths.getOptionsFilePath(), "w")
@@ -661,8 +695,8 @@ class ModRana:
       print("modRana: saving options failed:", e)
 
   def _loadOptions(self):
+    """load the persistent dictionary from file"""
     print("modRana: loading options")
-    success = False
     try:
       f = open(self.paths.getOptionsFilePath(), "r")
       newData = marshal.load(f)
@@ -671,8 +705,8 @@ class ModRana:
       for key in purgeKeys:
         if key in newData:
           del newData[key]
-      for k,v in newData.items():
-        self.set(k,v)
+      for k, v in newData.items():
+        self.set(k, v)
       success = True
     except Exception, e:
       print("modRana: exception while loading saved options:\n%s" % e)
@@ -705,13 +739,14 @@ class ModRana:
     userHomePath = os.getenv("HOME", "")
     profileFolderPath = os.path.join(userHomePath, modRanaProfileFolderName)
     # make sure it exists
-    createFolderPath(profileFolderPath)
+    utils.createFolderPath(profileFolderPath)
     # return it
     return profileFolderPath
 
   ## MAP LAYERS ##
-  """map layer information is important and needed by many modules during their initialization,
-  so it is handled here"""
+  # map layer information is important and needed by many modules during their initialization,
+  # so it is handled here
+
   def getMapLayers(self):
     return self.configs.getMapLayers()
 
@@ -719,11 +754,11 @@ class ModRana:
 
   def addTime(self, message):
     timestamp = time.time()
-    self.timing.append((message,timestamp))
+    self.timing.append((message, timestamp))
     return timestamp
 
   def addCustomTime(self, message, timestamp):
-    self.timing.append((message,timestamp))
+    self.timing.append((message, timestamp))
     return timestamp
 
   def reportStartupTime(self):
@@ -751,12 +786,19 @@ class ModRana:
     else:
       print("* timing list empty *")
 
+
 if __name__ == "__main__":
+  # change to folder where the main modRana file is located
+  # * this enables to run modRana with absolute path without adverse
+  # effect such as modRana not finding modules or
+  os.chdir(os.path.dirname(os.path.abspath(__file__)))
+  # add the modules folder to path, so that third-party modules (such as Upoints),
+  # that expect to be placed to path work correctly
+  sys.path.append('modules')
 
   # check if reload has been requested
-
   reloadArg = "--reload"
-  if len(sys.argv)>=3 and sys.argv[1] == reloadArg:
+  if len(sys.argv) >= 3 and sys.argv[1] == reloadArg:
     # following argument is path to the modRana main class we want to reload to,
     # optionally followed by any argument for the main class
     print(" == modRana Reloading == ")
@@ -764,7 +806,6 @@ if __name__ == "__main__":
     callArgs = [reloadPath]
     callArgs.extend(sys.argv[3:])
     subprocess.call(callArgs)
-
   else:
     program = ModRana()
 
