@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # An a generic netbook device module.
@@ -20,16 +19,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
-from base_device_module import deviceModule
+from base_device_module import DeviceModule
 
 def getModule(m,d,i):
-  return(device_netbook(m,d,i))
+  return DeviceNetbook(m,d,i)
 
-class device_netbook(deviceModule):
+class DeviceNetbook(DeviceModule):
   """A netbook modRana device-specific module"""
   
   def __init__(self, m, d, i):
-    deviceModule.__init__(self, m, d, i)
+    DeviceModule.__init__(self, m, d, i)
 
   def getDeviceIDString(self):
     return "netbook"
@@ -44,11 +43,4 @@ class device_netbook(deviceModule):
     return False
 
   def getSupportedGUIModuleIds(self):
-    return ["GTK", "QML"]
-    
-
-if(__name__ == "__main__"):
-  a = device_example({}, {})
-  a.update()
-  a.update()
-  a.update()
+    return ["GTK", "QML"]
\ No newline at end of file
