@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # A modRana GTK GUI class
@@ -36,6 +35,11 @@ from gtk import gdk
 
 from base_gui_module import GUIModule
 
+CLICK_DRAG_THRESHOLD = 1024
+# Adjust this to the length^2 of a
+# gerfingerpoken on your touchscreen
+# (1024 is for Freerunner, since it's very high resolution)
+
 def simplePythagoreanDistance(x1, y1, x2, y2):
   """convenience PyThagorean distance :)"""
   dx = x2 - x1
@@ -44,7 +48,7 @@ def simplePythagoreanDistance(x1, y1, x2, y2):
 
 
 def getModule(m, d, i):
-  return(GTKGUI(m, d, i))
+  return GTKGUI(m, d, i)
 
 
 class GTKGUI(GUIModule):
@@ -322,7 +326,7 @@ class GTKGUI(GUIModule):
 
 
 class MainWidget(gtk.Widget):
-  __gsignals__ = {\
+  __gsignals__ = {
     'realize': 'override',
     'expose-event': 'override',
     'size-allocate': 'override',
@@ -352,7 +356,6 @@ class MainWidget(gtk.Widget):
     self.timer1 = gobject.timeout_add(100, self.update, self) #default 100
     self.timer3 = None # will be used for timing long press events
 
-    #    self.mapRotationAngle = 0 # in radians
     self.notMovingSpeed = 1 # in m/s
 
     self.topWindow = None
@@ -404,15 +407,11 @@ class MainWidget(gtk.Widget):
     self.modrana.update()
 
   def get(self, key, defaultValue):
-    """
-    local alias for the modRana persistent dictionary get function
-    """
+    """local alias for the modRana persistent dictionary get function"""
     self.modrana.get(key, defaultValue)
 
   def set(self, key, value):
-    """
-    local alias for the modRana persistent dictionary set function
-    """
+    """local alias for the modRana persistent dictionary set function"""
     self.modrana.set(key, value)
 
   def _checkForRedrawCB(self, key, oldValue, newValue):
@@ -424,8 +423,8 @@ class MainWidget(gtk.Widget):
     """Make the window trigger a draw event.
        TODO: consider replacing this if porting pyroute to another platform"""
     self.set('needRedraw', False)
-    """ alter directly, no need to notificate
-    about returning the key to default state"""
+    # alter directly, no need to notificate
+    # about returning the key to the default state
 
     # record timestamp
     self.lastFullRedrawRequest = time.time()
@@ -444,7 +443,7 @@ class MainWidget(gtk.Widget):
 
     self.dragStartX = event.x
     self.dragStartY = event.y
-    #print "Pressed button %d at %1.0f, %1.0f" % (event.button, event.x, event.y)
+    #print("Pressed button %d at %1.0f, %1.0f" % (event.button, event.x, event.y))
 
     self.dragX = event.x
     self.dragY = event.y
@@ -455,23 +454,22 @@ class MainWidget(gtk.Widget):
   def moved(self, w, event):
     """Drag-handler"""
 
-    self.handleDrag(\
+    self.handleDrag(
       event.x,
       event.y,
       event.x - self.dragX,
       event.y - self.dragY,
       self.dragStartX,
       self.dragStartY,
-      event.time - self.lastPressEpoch)
+      event.time - self.lastPressEpoch
+    )
 
     self.dragX = event.x
     self.dragY = event.y
 
   def released(self, w, event):
-    """
-    mouse button has been released or
-    the tapping object has been lifted up from the touchscreen
-    """
+    """mouse button has been released or
+    the tapping object has been lifted up from the touchscreen"""
     self.pressInProgress = False
     self.pressLengthTimer = None
     msDuration = event.time - self.lastPressEpoch
@@ -485,8 +483,7 @@ class MainWidget(gtk.Widget):
     dx = event.x - self.dragStartX
     dy = event.y - self.dragStartY
     distSq = dx * dx + dy * dy
-    # Adjust this to the length^2 of a gerfingerpoken on your touchscreen (1024 is for Freerunner, since it's very high resolution)
-    if distSq < 1024:
+    if distSq < CLICK_DRAG_THRESHOLD:
       self.click(event.x, event.y, msDuration)
 
   def checkStillPressed(self, pressStartEpoch, pressStartTime, startX, startY):
@@ -495,15 +492,14 @@ class MainWidget(gtk.Widget):
     duration
     start coordinates
     currentCoordinates
-    if no press is in progress or another press already started, shut down the timer
-    """
+    if no press is in progress or another press already started, shut down the timer"""
 
-    """just to be sure, time out after 60 seconds
-    - provided the released signal is always called, this timeout might not be necessary,
-    but better be safe, than eat the whole battery if the timer is not terminated"""
+    # just to be sure, time out after 60 seconds
+    # - provided the released signal is always called, this timeout might not be necessary,
+    # but better be safe, than eat the whole battery if the timer is not terminated
     dt = (time.time() - pressStartTime) * 1000
     if dt > 60000:
-      print "long press timeout reached"
+      print("long press timeout reached")
       return False
 
     if pressStartEpoch == self.lastPressEpoch and self.pressInProgress:
@@ -520,9 +516,7 @@ class MainWidget(gtk.Widget):
       self.update()
 
   def handleDrag(self, x, y, dx, dy, startX, startY, msDuration):
-    """
-    handle dragging on the screen
-    """
+    """handle dragging on the screen"""
     # check if centering is on
     if self.modrana.get("centred", True):
       fullDx = x - startX
@@ -539,7 +533,14 @@ class MainWidget(gtk.Widget):
         # start simple map drag if its not already in progress
         menuName = self.modrana.get('menu', None)
         if menuName is None and not self.altMapDragInProgress:
-          self.altDragStart(x - startX, y - startY, dx, dy)
+          # don't start the alternative drag until
+          # the drag threshold value is reached
+          # otherwise, it would be almost impossible
+          # to click the on-screen buttons
+          totalDX, totalDY = startX-x, startY-y
+          distSq = totalDX * totalDX + totalDY * totalDY
+          if distSq > CLICK_DRAG_THRESHOLD:
+            self.altDragStart(x - startX, y - startY, dx, dy)
         elif self.altMapDragInProgress:
           self.altDragHandler(x - startX, y - startY, dx, dy)
       else:
@@ -565,20 +566,16 @@ class MainWidget(gtk.Widget):
     self.currentDrawMethod(cr, event)
     # enable redraw speed debugging
     if self.showRedrawTime:
-      print "Redraw took %1.2f ms" % (1000 * (time.clock() - start))
+      print("Redraw took %1.2f ms" % (1000 * (time.clock() - start)))
     self.lastFullRedraw = time.time()
 
   def getLastFullRedraw(self):
-    """
-    return when the GUI was last redrawn
-    """
+    """return when the GUI was last redrawn"""
     return self.lastFullRedraw
 
   def fullDrawMethod(self, cr, event):
-    """
-    this is the default drawing method
-    draws all layers and should be used together with full screen redraw
-    """
+    """this is the default drawing method
+    draws all layers and should be used together with full screen redraw"""
 
     # get all modules
     modules = self.modrana.getModules().values()
@@ -626,7 +623,9 @@ class MainWidget(gtk.Widget):
           for m in modules:
             m.drawMapOverlay(cr)
         except Exception, e:
-          print "modRana GTK main loop: an exception occurred:\n"
+          print("modRana GTK main loop: an exception occurred")
+          print(e)
+          print('modRana GTK main loop: traceback:')
           traceback.print_exc(file=sys.stdout) # find what went wrong
         cr.restore()
         cr.translate(-x, -y)
@@ -639,7 +638,9 @@ class MainWidget(gtk.Widget):
           for m in modules:
             m.drawMapOverlay(cr)
         except Exception, e:
-          print "modRana GTK main loop: an exception occurred:\n"
+          print("modRana GTK main loop: an exception occurred")
+          print(e)
+          print('modRana GTK main loop: traceback:')
           traceback.print_exc(file=sys.stdout) # find what went wrong
         for m in modules:
           m.drawScreenOverlay(cr)
@@ -663,7 +664,6 @@ class MainWidget(gtk.Widget):
     else:
       self.currentDrawMethod = method
 
-
   def defaultMethodBindings(self):
     """revert drag method bindings to default state"""
     self.altDragStart = None # alternative map drag enabler
@@ -702,7 +702,7 @@ class MainWidget(gtk.Widget):
     try:
       (sx, sy, w, h) = self.modrana.get('viewport', None)
     except TypeError:
-      print "gtk gui: viewport not available"
+      print("gtk gui: viewport not available")
       return
 
     # store current screen content in backing pixmap
@@ -745,7 +745,7 @@ class MainWidget(gtk.Widget):
 
   def do_realize(self):
     self.set_flags(self.flags() | gtk.REALIZED)
-    self.window = gdk.Window(\
+    self.window = gdk.Window(
       self.get_parent_window(),
       width=self.allocation.width,
       height=self.allocation.height,
@@ -776,7 +776,7 @@ class MainWidget(gtk.Widget):
     newW = allocation[2]
     newH = allocation[3]
 
-    print "GTK GUI: size allocation", allocation
+    print("GTK GUI: size allocation", allocation)
     # resize the backing pixmap
     self.initGCandBackingPixmap(self.allocation.width, self.allocation.height)
 
@@ -826,9 +826,9 @@ class MainWidget(gtk.Widget):
       #
       #
       ##      cr.paint()
-      ##      print cr.get_target()
-      ##      print cr.get_target().get_width()
-      ##      print cr.get_target().get_height()
+      ##      print(cr.get_target())
+      ##      print(cr.get_target().get_width())
+      ##      print(cr.get_target().get_height())
       #
       #    mapAndMapOverlayBuffer = self.getMapAndMapOverlayBuffer()
       #    if mapAndMapOverlayBuffer:
@@ -840,8 +840,8 @@ class MainWidget(gtk.Widget):
       #
       #    # enable redraw speed debugging
       #    if 'showRedrawTime' in self and self['showRedrawTime'] == True:
-      #      print "Redraw1 took %1.2f ms" % (1000 * (time.clock() - start))
-      #      print "Redraw2 took %1.2f ms" % (1000 * (time.clock() - start1))
+      #      print("Redraw1 took %1.2f ms" % (1000 * (time.clock() - start)))
+      #      print("Redraw2 took %1.2f ms" % (1000 * (time.clock() - start1)))
       #
       #  def getMapAndMapOverlayBuffer(self):
       #    if self.mapBuffer == None:
@@ -912,7 +912,7 @@ class MainWidget(gtk.Widget):
       #      for m in self.m.values():
       #        m.drawMapOverlay(cr1)
       #    except Exception, e:
-      #      print "modRana simple map: an exception occured:\n"
+      #      print("modRana simple map: an exception occured:\n")
       #      traceback.print_exc(file=sys.stdout) # find what went wrong
       #      self.stopSimpleMapDrag()
       #      return
