@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # A device independent display control module
@@ -18,25 +17,22 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
-from base_module import ranaModule
+from modules.base_module import RanaModule
 import time
 
 # only import GKT libs if GTK GUI is used
 from core import gs
 if gs.GUIString == "GTK":
-  import cairo
   import gtk
-  import pycha.line
 
 def getModule(m,d,i):
-  return(display(m,d,i))
+  return Display(m,d,i)
 
-class display(ranaModule):
+class Display(RanaModule):
   """A platform independent display device control module"""
   
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
-    gui = self.modrana.gui
+    RanaModule.__init__(self, m, d, i)
 
     self.fullscreen = False
     """according to documentation on:
@@ -86,13 +82,13 @@ class display(ranaModule):
   def enableRedraw(self,reason="not given"):
     """enable window redrawing"""
     self.modrana.gui.setRedraw(True)
-    print "display: redraw ON (%s)" % reason
+    print("display: redraw ON (%s)" % reason)
     self.set('needRedraw',True) # make sure the screen is refreshed
 
   def disableRedraw(self,reason="not given"):
     """disable window redrawing"""
     self.modrana.gui.setRedraw(False)
-    print "display: redraw OFF (%s)" % reason
+    print("display: redraw OFF (%s)" % reason)
 
   def windowStateChangedCallback(self, window, event):
     if event.new_window_state == gtk.gdk.WINDOW_STATE_ICONIFIED:
@@ -102,7 +98,7 @@ class display(ranaModule):
     else:
       self.enableRedraw(reason="window not hidden or minimised")
 
-  def visibilityChangedCallback(self,window, event):
+  def visibilityChangedCallback(self, window, event):
     if event.state == gtk.gdk.VISIBILITY_FULLY_OBSCURED:
       self.disableRedraw(reason="window is obscured")
     else:
@@ -114,12 +110,12 @@ class display(ranaModule):
           self.modrana.gui.setFullscreen(False)
           self.fullscreen = False
           self.menusSetFullscreen(self.fullscreen)
-          print "going out of fullscreen"
+          print("going out of fullscreen")
     else:
       self.modrana.gui.setFullscreen(True)
       self.fullscreen = True
       self.menusSetFullscreen(self.fullscreen)
-      print "going to fullscreen"
+      print("going to fullscreen")
 
   def menusSetFullscreen(self, value):
     """update the cached value in the menus module"""
@@ -285,9 +281,3 @@ class display(ranaModule):
         if self.checkMethod:
           self.checkMethod() # call the check method
         self.lastCheckConditions = currentTime
-
-if(__name__ == "__main__"):
-  a = example({}, {})
-  a.update()
-  a.update()
-  a.update()
