@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # A modRana module providing various kinds of information.
@@ -18,17 +17,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
-from base_module import ranaModule
+from modules.base_module import RanaModule
 import os
 
 def getModule(m,d,i):
-  return(info(m,d,i))
+  return Info(m,d,i)
 
-class info(ranaModule):
+class Info(RanaModule):
   """A modRana information handling module"""
   
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
+    RanaModule.__init__(self, m, d, i)
     self.versionString = "unknown version"
     currentVersionString = self.modrana.paths.getVersionString()
     if currentVersionString is not None:
@@ -87,7 +86,6 @@ class info(ranaModule):
     if menuName == 'infoAbout':
       menus = self.m.get('menu', None)
       if menus:
-        nop = "set:menu:info#infoAbout"
         button1 = ('Discussion', 'generic', "ms:menu:openUrl:%s" % self.getDiscussionUrls()[0][0])
         button2 = ('Donate', 'generic', "ms:menu:openUrl:%s" % self.getPayPalUrl())
         web = " <u>www.modrana.org</u> "
@@ -95,9 +93,3 @@ class info(ranaModule):
         text = "modRana version:\n\n%s\n\n\n\nFor questions or feedback,\n\ncontact the <b>modRana</b> project:\n\n%s\n\n%s\n\n" % (self.versionString,web,email)
         box = (text ,"ms:menu:openUrl:http://www.modrana.org")
         menus.drawThreePlusOneMenu(cr, 'infoAbout', 'set:menu:info', button1, button2, box)
-
-if(__name__ == "__main__"):
-  a = example({}, {})
-  a.update()
-  a.update()
-  a.update()
