@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # Sample of a Rana module.
@@ -19,26 +18,25 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
 #from dbus.service import Object
+from modules.base_module import RanaModule
+from core  import geo
+from core import utils
 import math
-from base_module import ranaModule
-import geo
 import os
-import modrana_utils
 import glob
 import cPickle
 import shutil
-import modrana_utils
 from time import clock
 from time import gmtime, strftime
 
 def getModule(m,d,i):
-  return(loadTracklogs(m,d,i))
+  return LoadTracklogs(m,d,i)
 
-class loadTracklogs(ranaModule):
+class LoadTracklogs(RanaModule):
   """A sample pyroute module"""
   
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
+    RanaModule.__init__(self, m, d, i)
     self.tracklogs = {} # dictionary of all loaded tracklogs, path is the key
     #self.set('tracklogs', self.tracklogs) # now we make the list easily acessible to other modules
     self.cache = {}
@@ -77,7 +75,7 @@ class loadTracklogs(ranaModule):
           print("** Assuring tracklog cache consistency")
           self.save()
           self.cleanCache()
-          print "** Tracklog cache consistency assured"
+          print("** Tracklog cache consistency assured")
 #    elif message == 'renameActiveTracklog':
 #      activeTracklog = self.getActiveTracklog()
 #      if activeTracklog:
@@ -94,7 +92,7 @@ class loadTracklogs(ranaModule):
       return None # tracklog folder path is unknown
     else:
       TFSubPath = os.path.join(tracklogFolderPath, subPath)
-      modrana_utils.createFolderPath(TFSubPath)
+      utils.createFolderPath(TFSubPath)
       return TFSubPath
 
   def _createBasicFolderStructure(self):
@@ -108,7 +106,7 @@ class loadTracklogs(ranaModule):
     tfp = self.modrana.paths.getTracklogsFolderPath()
     examplesDestinationPath = os.path.join(tfp,'examples')
     if not os.path.exists(examplesDestinationPath):
-      modrana_utils.createFolderPath(examplesDestinationPath)
+      utils.createFolderPath(examplesDestinationPath)
       print(' ** loadTracklogs: copying example tracklogs')
       examplesSourcePath = 'data/tracklog_examples'
        # copy all files from this folder
@@ -124,23 +122,22 @@ class loadTracklogs(ranaModule):
 
   def loadCache(self):
     # unpickle the cache from file
-    print "** Loading tracklog cache"
+    print("** Loading tracklog cache")
     start = clock()
     try:
       f = open(self.getTracklogCachePath(), 'r')
       self.cache = cPickle.load(f)
       f.close()
-    except:
-      print "loadTracklogs: loading cache from file failed"
+    except Exception, e:
+      print("loadTracklogs: loading cache from file failed")
+      print(e)
       self.cache = {}
-    print "** Loading tracklog cache took %1.2f ms" % (1000 * (clock() - start))
+    print("** Loading tracklog cache took %1.2f ms" % (1000 * (clock() - start)))
 
   def cleanCache(self):
     """remove files that are not present from the cache"""
     paths = self.tracklogPathList
     garbage = filter(lambda x: x not in paths, self.cache)
-
-
     for g in garbage:
       del self.cache[g]
 
@@ -252,7 +249,7 @@ class loadTracklogs(ranaModule):
         filename = os.path.split(file)[1]
         lastModifiedEpochSecs = os.path.getmtime(path)
         lastModified = strftime("%d.%m.%Y %H:%M:%S",gmtime(lastModifiedEpochSecs))
-        size = modrana_utils.bytes2PrettyUnitString(os.path.getsize(path))
+        size = utils.bytes2PrettyUnitString(os.path.getsize(path))
         extension = os.path.splitext(path)[1]
         cat = folder
         item={'path':path,
@@ -317,10 +314,10 @@ class loadTracklogs(ranaModule):
 #
 #      self.cache = cache
 #    except:
-#      print "loadTracklogs: loading cache from file failed"
+#      print("loadTracklogs: loading cache from file failed")
 #      self.cache = {}
 #
-#    print "Loading from cache took %1.2f ms" % (1000 * (clock() - start))
+#    print("Loading from cache took %1.2f ms" % (1000 * (clock() - start)))
 #
 #    files = []
 #    if os.path.exists(self.tracklogFolder):
@@ -328,8 +325,8 @@ class loadTracklogs(ranaModule):
 #      files = filter(lambda x: x != '.svn', files)
 #
 #
-#      print self.tracklogFolder
-#      print os.path.exists(self.tracklogFolder)
+#      print(self.tracklogFolder)
+#      print(os.path.exists(self.tracklogFolder))
 #      for file in files:
 #        try:
 #          self.loadTracklog(self.tracklogFolder + file)
@@ -338,16 +335,16 @@ class loadTracklogs(ranaModule):
 #
 #    self.cleanCache(files)
 #    self.save()
-#    print "Loading tracklogs took %1.2f ms" % (1000 * (clock() - start))
+#    print("Loading tracklogs took %1.2f ms" % (1000 * (clock() - start)))
 
   def save(self):
     try:
-      print self.getTracklogCachePath()
+      print(self.getTracklogCachePath())
       f = open(self.getTracklogCachePath(), 'w')
       cPickle.dump(self.cache, f)
       f.close()
     except Exception, e:
-      print "loadTracklogs: can't store tracklog data to cache, tracklogs will be loaded from files next time"
+      print("loadTracklogs: can't store tracklog data to cache, tracklogs will be loaded from files next time")
       print("exception: %r" % e)
 
   def setPathAsActiveTracklog(self, path):
@@ -520,7 +517,7 @@ class loadTracklogs(ranaModule):
 
 
 
-class tracklog():
+class Tracklog():
   """A basic class representing a tracklog."""
   def __init__(self, trackpointsList, filename, type):
     self.trackpointsList = trackpointsList # here will be the actual list of trackpoints
@@ -563,11 +560,11 @@ class tracklog():
     return None
 
 
-class GPXTracklog(tracklog):
+class GPXTracklog(Tracklog):
   """A class representing a GPX tracklog."""
   def __init__(self, trackpointsList, filename, type, cache, save):
-    tracklog.__init__(self, trackpointsList, filename, type)
-    tracklog.type = 'GPX'
+    Tracklog.__init__(self, trackpointsList, filename, type)
+    Tracklog.type = 'GPX'
     self.routeInfo = None # a dictionary for storing route information
     # TODO: set this automatically
 
@@ -603,7 +600,7 @@ class GPXTracklog(tracklog):
       rawClusters = geo.clusterTrackpoints(trackpointsList, clusterDistance) # we cluster the points
       for cluster in rawClusters: # now we find for each cluster a circle encompassing all points
         (centreX,centreY,radius) = geo.circleAroundPointCluster(cluster)
-        self.clusters.append(clusterOfPoints(cluster, centreX, centreY, radius))
+        self.clusters.append(ClusterOfPoints(cluster, centreX, centreY, radius))
 
       self.checkElevation()
 
@@ -664,7 +661,7 @@ class GPXTracklog(tracklog):
     f = open(self.filename, "w") # open the old file
     xmlTree = self.trackpointsList.export_gpx_file() # get the element tree
     xmlTree.write(f) # overwrite the old file with the new structure
-    print "%s has been replaced by the current in memory version" % self.filename
+    print("%s has been replaced by the current in memory version" % self.filename)
     del self.cache[self.filename] # the file has been modified, so it must be cached again
     self.save() # save the cache to disk
 
@@ -679,7 +676,7 @@ class CacheItem():
     self.routeInfo = routeInfo
     self.perElevList = perElevList
 
-class clusterOfPoints():
+class ClusterOfPoints():
   """A basic class representing a cluster of nearby points."""
   def __init__(self, pointsList, centreX, centreY, radius):
     self.pointsList = pointsList # points in the cluster
@@ -687,10 +684,3 @@ class clusterOfPoints():
     self.centreX = centreX
     self.centreY = centreY
     self.radius = radius #radius of the circle
-
-
-if(__name__ == "__main__"):
-  a = example({}, {})
-  a.update()
-  a.update()
-  a.update()
