@@ -17,17 +17,21 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
 from __future__ import with_statement # for python 2.5
-from base_module import ranaModule
+from modules.base_module import RanaModule
 from time import *
 from core import gs
+from core.signal import Signal
 
-def getModule(m,d,i):
-  return(Location(m,d,i))
 
-class Location(ranaModule):
+def getModule(m, d, i):
+  return Location(m, d, i)
+
+
+class Location(RanaModule):
   """Supplies position info from a position source"""
+
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
+    RanaModule.__init__(self, m, d, i)
     self.tt = 0
     self.connected = False
     self.set('speed', None)
@@ -37,6 +41,8 @@ class Location(ranaModule):
     self.status = "Unknown"
     self.enabled = False
     self.provider = None
+    self.startSignal = Signal()
+    self.stopSignal = Signal()
 
     # check if the device handles location by itself
     if not self.modrana.dmod.handlesLocation():
@@ -44,16 +50,21 @@ class Location(ranaModule):
       if method == "qt_mobility":
         print(" @ location: using Qt Mobility")
         import qt_mobility
+
         self.provider = qt_mobility.QtMobility(self)
       else: # GPSD
         print(" @ location: using GPSD")
         import gps_daemon
+
         self.provider = gps_daemon.GPSD(self)
 
+    # watch if debugging needs to be enabled
+    self.modrana.watch("gpsDebugEnabled", self._debugCB, runNow=True)
+
   def firstTime(self):
     # periodic screen redraw
     if self.modrana.dmod.getLocationType() in ("gpsd", "liblocation"):
-      print "location: starting GPSD 1 second timer"
+      print("location: starting GPSD 1 second timer")
       # start screen update 1 per second screen update
       # TODO: event based redrawing
       cron = self.m.get('cron', None)
@@ -67,7 +78,7 @@ class Location(ranaModule):
   def _screenUpdateCB(self):
     """update the screen and also GPSD location if enabled
     TODO: more efficient screen updates"""
-#    print("location: screen update")
+    #    print("location: screen update")
 
     # only try to update position info if
     # location is enabled
@@ -96,19 +107,23 @@ class Location(ranaModule):
       if sFromLastRequest > 0.85:
         self.set('needRedraw', True)
 
+  def _debugCB(self, key, oldValue, newValue):
+    if self.provider:
+      self.provider.setDebug(newValue)
+
   def handleMessage(self, message, type, args):
     if message == "setPosLatLon" and type == "ml":
       if args and len(args) == 2:
         lat = float(args[0])
         lon = float(args[1])
-        print "gps:setting current position to: %f,%f" % (lat,lon)
-        self.set('pos',(lat,lon))
+        print("gps:setting current position to: %f,%f" % (lat, lon))
+        self.set('pos', (lat, lon))
     elif message == "checkGPSEnabled":
-        state = self.get('GPSEnabled', True)
-        if state == True:
-          self.startLocation()
-        elif state == False:
-          self.stopLocation()
+      state = self.get('GPSEnabled', True)
+      if state == True:
+        self.startLocation()
+      elif state == False:
+        self.stopLocation()
     elif message == "gpsdCheckVerboseDebugEnabled":
       self._checkVerbose()
 
@@ -121,21 +136,21 @@ class Location(ranaModule):
       self.set('fix', 0)
       # wait for valid data
       return
-    (lat,lon) = fix.position
+    (lat, lon) = fix.position
     fixClass = 2 # 2D data
 
     # position
-    self.set('pos', (lat,lon))
+    self.set('pos', (lat, lon))
     self.set('pos_source', 'GPSD')
     # bearing
     self.set('bearing', float(fix.bearing))
     # speed
-#    if speed != None:
-#      # normal gpsd reports speed in knots per second
-#      gpsdSpeed = self.get('gpsdSpeedUnit', 'knotsPerSecond')
-#      if gpsdSpeed == 'knotsPerSecond':
-#        # convert to meters per second
-#        speed = float(speed) * 0.514444444444444 # knots/sec to m/sec
+    #    if speed != None:
+    #      # normal gpsd reports speed in knots per second
+    #      gpsdSpeed = self.get('gpsdSpeedUnit', 'knotsPerSecond')
+    #      if gpsdSpeed == 'knotsPerSecond':
+    #        # convert to meters per second
+    #        speed = float(speed) * 0.514444444444444 # knots/sec to m/sec
     if fix.speed is not None:
       self.set('metersPerSecSpeed', fix.speed)
       self.set('speed', fix.speed * 3.6)
@@ -157,7 +172,6 @@ class Location(ranaModule):
     # 3 - 3D
     self.set("fix", fixClass)
 
-
     """always set this key to current epoch once the location is updated
     so that modules can watch it and react on position updates"""
     self.set('locationUpdated', time())
@@ -165,21 +179,25 @@ class Location(ranaModule):
   def getFix(self):
     return self.provider.getFix()
 
-  def startLocation(self):
+  def startLocation(self, startMainLoop=False):
     """start location - device based or gpsd"""
+    # send the location start signal
+    self.startSignal()
     if not self.enabled:
-      print "location: enabling location"
+      print("location: enabling location")
       if self.modrana.dmod.handlesLocation():
-        self.modrana.dmod.startLocation()
+        self.modrana.dmod.startLocation(startMainLoop=startMainLoop)
       else:
-        self.provider.start()
+        self.provider.start(startMainLoop=startMainLoop)
       self.enabled = True
     else:
       print('location: location already enabled')
 
   def stopLocation(self):
     """stop location - device based or gpsd"""
-    print "location: disabling location"
+    # send the location stop signal
+    self.stopSignal()
+    print("location: disabling location")
     if self.modrana.dmod.handlesLocation():
       self.modrana.dmod.stopLocation()
     else:
@@ -190,4 +208,8 @@ class Location(ranaModule):
     try:
       self.stopLocation()
     except Exception, e:
-      print "location: stopping location failed", e
+      print("location: stopping location failed", e)
+
+  def _checkVerbose(self):
+    if self.provider:
+      self.provider._checkVerbose()
\ No newline at end of file
