@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # Handle option menus
@@ -18,19 +17,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
-from base_module import ranaModule
-import marshal
-import modrana_utils
+from modules.base_module import RanaModule
+from core import utils
+
+# identifies item as a group
+GROUP_IDENTIFIER = "groupIdentifier"
 
 def getModule(m, d, i):
-  return(Options(m, d, i))
+  return Options(m, d, i)
 
 
-class Options(ranaModule):
+class Options(RanaModule):
   """Handle options"""
 
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
+    RanaModule.__init__(self, m, d, i)
     self.options = {}
     """for fast searching defaults for corresponding keys"""
     self.keyDefault = {}
@@ -38,7 +39,7 @@ class Options(ranaModule):
     # profile folder
     self.profileFolderPath = self.modrana.getProfilePath()
     # check the profile path and create the folders if necessary
-    modrana_utils.createFolderPath(self.profileFolderPath)
+    utils.createFolderPath(self.profileFolderPath)
 
     # load persistent options
     #    self.load()
@@ -143,6 +144,27 @@ class Options(ranaModule):
     }
     self.addOption(title, variable, choices, group, None)
 
+  def _generateNestedItems(self, inputList, variable, backAction, fakeMode=None):
+    """generate item tuples for nested item menus"""
+    menuItems = [] # an ordered list of all the menu items
+    itemDict = {} # for easily assigning keys to labels
+    id = 1 # id 0 is the escape button
+
+    for value, name, icon, action in inputList:
+      if value == "groupIdentifier": # this item is a group
+        item = self.menuModule.generateItem("#%s" % name, "generic", action)
+      else: # this item is a button that sets a value
+        if fakeMode is None: # just use the current mode
+          item = self.menuModule.generateItem("#%s" % name, "generic",
+            "setWithCurrentMode:%s:%s|%s" % (variable, value, backAction))
+        else:# use a fake mode (used for per mode option state list)
+          item = self.menuModule.generateItem("#%s" % name, "generic",
+            "setWithMode:%s:%s:%s|%s" % (fakeMode, variable, value, backAction))
+      menuItems.append(item)
+      itemDict[value] = (name, id)
+      id += 1
+    return menuItems, itemDict
+
   def _generateItems(self, valueNameList, variable, backAction, fakeMode=None):
     menuItems = [] # an ordered list of all the menu items
     itemDict = {} # for easily assigning keys to labels
@@ -164,7 +186,7 @@ class Options(ranaModule):
 
     #NOTE: for the value - name mapping to work correctly, the value must be a string
     # -> this is caused by the value being sent as a string once one of the items is clicked
-    # -> if the value, is not string, just the raw value voul be shown
+    # -> if the value, is not string, just the raw value value be shown
     # Example:
     # (100, "100%") will show 100
     # ('100', "100%") will show 100%
@@ -200,6 +222,85 @@ class Options(ranaModule):
       choices['noToolsIcon'] = True # disable the tools in the per mode state list
     self.addOption(title, variable, choices, group, default)
 
+  def addNestedItemsOption(self, title, variable, items, group, default=None, fakeMode=None, preAction=None):
+    """add an option, that opens and item selection menu with groups"""
+
+    #NOTE: for the value - name mapping to work correctly, the value must be a string
+    # -> this is caused by the value being sent as a string once one of the items is clicked
+    # -> if the value, is not string, just the raw value value be shown
+    # Example:
+    # (100, "100%") will show 100
+    # ('100', "100%") will show 100%
+
+    # the back action returns back to the group from which the menu was opened
+    backAction = "set:menu:options#%s" % group
+
+    # create submenus for the groups & a toplevel menu
+
+    """NOTE: for the returning back to the group to work correctly,
+    the menu is stored under a key combined from the variable and group names"""
+    storageKey = self._getItemsOptionStorageKey(group, variable, fakeMode=fakeMode)
+
+    groupIndex = 1 # 0 is the back button
+    topLevel = []
+    itemDict = {}
+    tempItemDict = {}
+    for item in items:
+      value, name, icon, action = item["item"]
+      if item.get('group', None): # this is a group
+        subMenuItems = item.get('group', [])
+        # create per-group submenu
+        menuItems, tempItemDict = self._generateNestedItems(subMenuItems, variable, backAction, fakeMode=fakeMode)
+        groupStorageKey = "%s_%d" % (storageKey, groupIndex)
+        groupBackAction = "set:menu:%s" % storageKey
+        self._setItemsAsMenu(groupStorageKey, menuItems, groupBackAction)
+        # update value -> name mapping with correct group and subgroup IDs
+        for layerKey in tempItemDict.keys():
+          label = tempItemDict[layerKey][0]
+          subId = tempItemDict[layerKey][1]
+          # label, toplevel id, group id
+          # NOTE: group level highlighting is not yet implemented
+          tempItemDict[layerKey] = (label, groupIndex, subId)
+        # TODO: highlighting inside groups
+
+        itemDict.update(tempItemDict)
+
+        # override the action for the toplevel group button
+        # to point to the group menu
+        action = "set:menu:%s" % groupStorageKey
+        groupIndex+=1
+      # add the toplevel button
+      topLevel.append((value, name, icon, action))
+    # add the toplevel menu
+    menuItems, tempItemDict = self._generateNestedItems(topLevel, variable, backAction, fakeMode=fakeMode)
+    self._setItemsAsMenu(storageKey, menuItems, backAction)
+    # update value -> name mapping for tomplevel buttons
+    itemDict.update(tempItemDict)
+
+    # also store in the local options structure
+    choices = {"type": "selectOneItem",
+               'label': "",
+               'description': "",
+               'default': default,
+               'items': items,
+               'itemDict': itemDict,
+               'storageKey': storageKey,
+               'preAction': preAction # send this message before entering the menu
+    }
+    # this means we are probably showing the option in the per mode state list
+    if fakeMode is not None:
+      choices['mode'] = fakeMode
+      choices['noToolsIcon'] = True # disable the tools in the per mode state list
+    self.addOption(title, variable, choices, group, default)
+
+  def _setItemsAsMenu(self, storageKey, menuItems, backAction, wideButtons=True):
+    """create a new item menu (or overwrite an existing one) and register it in the
+    menu Module"""
+    menu = self.menuModule.getClearedMenu(backAction)
+    menu = self.menuModule.addItemsToThisMenu(menu, menuItems)
+    self.menuModule.addItemMenu(storageKey, menu, wideButtons=wideButtons)
+
+
   def _getItemsOptionStorageKey(self, group, variable, fakeMode=None):
     """return menu name for the special item selection itemized menu
     """
@@ -229,7 +330,7 @@ this is needed for the item tools menu to know where to return"""
       self.options[group][2].append(newOption)
       self.keyDefault[variable] = default
     else:
-      print "options: group %s does not exist, call addGroup to create it first" % group
+      print("options: group %s does not exist, call addGroup to create it first" % group)
 
   def addRawOption(self, optionData):
     """add a raw option to options
@@ -251,7 +352,7 @@ this is needed for the item tools menu to know where to return"""
         self.options[group][2].append(optionData)
         self.keyDefault[variable] = default
       else:
-        print "options: group %s does not exist, can't add a raw option to it" % group
+        print("options: group %s does not exist, can't add a raw option to it" % group)
 
   def removeOption(self, categoryId, groupId, variable):
     """remove an option given by group and variable name"""
@@ -264,7 +365,7 @@ this is needed for the item tools menu to know where to return"""
       if variable in self.keyDefault:
         del self.keyDefault[variable]
     else:
-      print "options: group %s does not exist, so option with variable %s can not be removed" % (group, variable)
+      print("options: group %s does not exist, so option with variable %s can not be removed" % (group, variable))
 
   def getOption(self, groupID, index):
     """get a options item from a given group by its index"""
@@ -272,11 +373,11 @@ this is needed for the item tools menu to know where to return"""
       try:
         return self.options[groupID][2][index]
       except IndexError:
-        print "options: group %s has no index %d, so this option can not be returned" % (groupID, index)
+        print("options: group %s has no index %d, so this option can not be returned" % (groupID, index))
         return False
 
     else:
-      print "options: group %s does not exist, so option with index %d can not be returned" % (groupID, index)
+      print("options: group %s does not exist, so option with index %d can not be returned" % (groupID, index))
       return False
 
   def getKeyDefault(self, key, default=None):
@@ -309,21 +410,49 @@ this is needed for the item tools menu to know where to return"""
 
     # ** map layers
     group = addGroup("Map layers", "map_layers", catMap, "generic")
+
+    defaultBA = "set:menu:options" # default back action
     layers = self.modrana.getMapLayers()
-    layerNameKey = []
-    for key in layers.keys():
-      name = layers[key]['label']
-      layerNameKey.append((key, name))
-    layerNameKey.sort()
-    layerNameKey.append((None, "Empty layer"))
-    addItems("Main map", "layer", layerNameKey, group, "mapnik")
+    groups = self.modrana.configs.getMapGroups()
+    layerStructure= []
+
+    # assign layers to groups
+    for key in groups.keys():
+      layerGroup = groups[key]
+      name = layerGroup.get('label', "group")
+      icon = layerGroup.get('icon', "generic")
+      # list all layers for this group
+      groupLayers = filter(lambda x: layers[x].get("group", None) == key, layers.keys())
+      # layer keys to list of layers
+      groupLayers = map(lambda x:
+      (x, layers[x]['label'], layers[x]['icon'], defaultBA)
+        ,groupLayers)
+      # sort them alphabetically
+      groupLayers.sort()
+      # append their counter to the group name
+      name = "%s (%d)" % (name, len(groupLayers))
+
+      layerStructure.append({"item":(GROUP_IDENTIFIER, name, icon, defaultBA),
+                             "group":groupLayers})
+
+    # append layers without group right after groups in the list
+    nonGroupLayers = filter(lambda x: layers[x].get("group", None) is None, layers.keys())
+    nonGroupLayers = map(lambda x:
+    {'item':(x, layers[x]['label'], layers[x]['icon'], defaultBA)},
+      nonGroupLayers)
+    nonGroupLayers.sort()
+    layerStructure.extend(nonGroupLayers)
+    # add empty layer
+    layerStructure.append({'item':(None, "Empty layer", "generic", defaultBA)})
+    # add the option
+    self.addNestedItemsOption("Main map", "layer", layerStructure, group, "mapnik")
 
     # ** Overlay
     group = addGroup("Map overlay", "map_overlay", catMap, "generic")
     addBoolOpt("Map as overlay", "overlay", group, False)
 
-    addItems("Main map", "layer", layerNameKey, group, "mapnik")
-    addItems("Background map", "layer2", layerNameKey, group, "cycle")
+    self.addNestedItemsOption("Main map", "layer", layerStructure, group, "mapnik")
+    self.addNestedItemsOption("Background map", "layer2", layerStructure, group, "cycle")
 
     addOpt("Transparency ratio", "transpRatio",
       [("0.25,1", "overlay:25%"),
@@ -391,6 +520,28 @@ this is needed for the item tools menu to know where to return"""
       group,
       defaultMode)
 
+    # ** map filtering
+    group = addGroup("Filters", "map_filtering", catMap, "generic")
+    addOpt("Negative", "invertMapTiles",
+      [(False, "disabled"),
+       (True, "enabled"),
+       ('withNightTheme', "with night theme")],
+      group,
+      False)
+
+    # ** map grid
+    group = addGroup("Grid", "map_grid", catMap, "generic")
+    addBoolOpt("Show grid", "drawMapGrid", group, False)
+    addOpt("Grid color", "mapGridColor",
+      [("white", '<span color="white">white</span>'),
+       ("black", '<span color="black">black</span>'),
+       ("red", '<span color="red">red</span>'),
+       ("green", '<span color="green">green</span>'),
+       ("blue", '<span color="blue">blue</span>')],
+      group,
+      "blue")
+    addBoolOpt("Labels", "mapGridLabels", group, True)
+
     # ** tile storage
     group = addGroup("Tile storage", "tile_storage", catMap, "generic")
     addOpt("Tile storage", "tileStorageType",
@@ -889,8 +1040,7 @@ this is needed for the item tools menu to know where to return"""
     locationType = self.dmod.getLocationType()
     if locationType == 'liblocation':
       addBoolOpt("Show N900 GPS-fix", "n900GPSDebug", group, False)
-    if locationType == 'gpsd':
-      addBoolOpt("Show gpsd debug output", "gpsdDebugVerbose", group, False, "location:gpsdCheckVerboseDebugEnabled")
+    addBoolOpt("GPS debug messages", "gpsDebugEnabled", group, False)
 
 
 
@@ -1064,7 +1214,7 @@ this is needed for the item tools menu to know where to return"""
   def handleTextEntryResult(self, key, result):
     (type, variable) = key.split("_", 1)
     if type == "editVariable":
-      print "editing variable: %s with: %s" % (variable, result)
+      print("editing variable: %s with: %s" % (variable, result))
       self.set(variable, result)
 
   def drawMenu(self, cr, menuName, args=None):
@@ -1151,7 +1301,8 @@ this is needed for the item tools menu to know where to return"""
             default = choices['default']
             value = self.get(variable, default, mode=mode)
             # show label for the given value
-            valueDescription, highlightId = choices['itemDict'].get(value, (value, None))
+            highlight = choices['itemDict'].get(value, (value, None))
+            valueDescription, highlightId = highlight[0], highlight[1]
             # if no description is found, just display the value
             valueDescription = "<tt><b>%s</b></tt>" % valueDescription
 
@@ -1219,7 +1370,7 @@ this is needed for the item tools menu to know where to return"""
             None,
             "generic", # background for a 3x1 icon
             "")
-          # due to the button on the righ, register a slightly smaller area
+          # due to the button on the right, register a slightly smaller area
           clickHandler.registerXYWH(x4, y, w - smallButtonW, dy, onClick)
 
           # draw mode specific combined toggle & indicator
@@ -1261,8 +1412,4 @@ this is needed for the item tools menu to know where to return"""
 
           # in corner: row number
           indexX = x4 + w * 0.90 - smallButtonW
-          self.menuModule.showText(cr, "%d/%d" % (index + 1, numItems), indexX, y + dy * 0.07, w * 0.10 - border, 20)
-
-if(__name__ == "__main__"):
-  a = options({}, {'viewport': (0, 0, 600, 800)})
-  a.firstTime()
+          self.menuModule.showText(cr, "%d/%d" % (index + 1, numItems), indexX, y + dy * 0.07, w * 0.10 - border, 20)
\ No newline at end of file
