@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #---------------------------------------------------------------------------
 # Finds routes using Google Direction (and possibly other services in the future).
@@ -18,7 +17,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
-from modules.base_module import ranaModule
+from modules.base_module import RanaModule
 
 import os
 import sys
@@ -27,32 +26,42 @@ import re
 import csv
 import traceback
 import unicodedata
-import modules.way as way
+import core.way as way
+import core.geo as geo
 #from time import clock
 import time
 
 DIRECTIONS_FILTER_CSV_PATH = 'data/directions_filter.csv'
 
+# routing error codes
 ROUTING_SUCCESS = 0
 ROUTING_NO_DATA = 1 # failed to load routing data
 ROUTING_LOAD_FAILED = 2 # failed to load routing data
 ROUTING_LOOKUP_FAILED = 3 # failed to locate nearest way/edge
 ROUTING_ROUTE_FAILED = 4 # failed to compute route
 
+# OSD menu states
+OSD_EDIT = 1 # route editing buttons
+OSD_CURRENT_ROUTE = 2 # a single button that triggers the options menu
+OSD_ROUTE_OPTIONS = 3 # buttons that go to the edit or info menus
+
 def getModule(m, d, i):
-  return(route(m, d, i))
+  return Route(m, d, i)
 
 
-class route(ranaModule):
+class Route(RanaModule):
   """Routes"""
 
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
+    RanaModule.__init__(self, m, d, i)
     self._goToInitialState()
     self.routeRequestSentTimestamp = None
     self.once = True
     self.entry = None
 
+    # tracks the state of the onscreen routing menu
+    self.osdMenuState = None
+
     self.set('startPos', None)
     self.set('endPos', None)
 
@@ -71,16 +80,24 @@ class route(ranaModule):
     self.routeRequestSentTimestamp = None
     self.pxpyRoute = [] # route in screen coordinates
     self.directions = [] # directions object
-    self.duration = None # in seconds
+    self.set('midText', [])
+    self.durationString = None # in seconds
     self.start = None
     self.destination = None
     self.startAddress = None
     self.destinationAddress = None
     self.text = None
+    self.selectManyPoints = False
+    # disable OSD menu
+    self.osdMenuState = None
+    # discern between normal routing with waypoints
+    # and handmade routing
+    self.handmade = False
     self.selectTwoPoints = False
     self.selectOnePoint = False
 
     self.expectStart = False
+    self.expectMiddle = False # handmade
     self.expectEnd = False
 
     self.routeDetailGeocodingTriggered = False
@@ -94,6 +111,7 @@ class route(ranaModule):
       self._goToInitialState()
       self.set('startPos', None)
       self.set('endPos', None)
+      self.set('middlePos', []) # handmade
 
       # stop Turn-by-turn navigation, that can be possibly running
       self.sendMessage('turnByTurn:stop')
@@ -120,6 +138,29 @@ class route(ranaModule):
       self.expectStart = False
       self.set('needRedraw', True) # refresh the screen to show the new point
 
+    elif message == 'expectMiddle': # handmade
+      self.expectMiddle = True # handmade
+      self.set('needRedraw', True) # we need to show the changed buttons # handmade
+
+    elif message == 'setMiddle': # handmade
+      proj = self.m.get('projection', None)
+      if proj and self.expectMiddle:
+        lastClick = self.get('lastClickXY', None)
+        (x, y) = lastClick
+        """
+        x and y must be floats, otherwise strange rounding errors occur, when converting to lat lon coordinates
+        """
+        (lat, lon) = proj.xy2ll(x, y)
+        middlePos=self.get('middlePos', [])
+        middlePos.append((lat,lon,None, ""))
+        self.set('middlePos', middlePos)
+        # if in handmade input mode,
+        # also ask for instructions for the given point
+        if self.handmade:
+          self.sendMessage('route:middleInput')
+      self.expectMiddle = False
+      self.set('needRedraw', True) # refresh the screen to show the new point
+
     elif message == 'expectEnd':
       self.expectEnd = True
       self.set('needRedraw', True) # we need to show the changed buttons
@@ -142,42 +183,80 @@ class route(ranaModule):
       self.expectEnd = False
       self.set('needRedraw', True) # refresh the screen to show the new point
 
-    elif message == "selectTwoPoints":
+    elif message == "handmade":
       self.set('startPos', None)
+      self.set('middlePos', [])
       self.set('endPos', None)
       self.selectOnePoint = False
       self.selectTwoPoints = True
+      self.selectManyPoints = True
+      self.handmade = True
+      self.osdMenuState = OSD_EDIT
+      print("HANDMADE")
+      print(self.handmade)
+
+    elif message == "selectTwoPoints":
+      self.set('startPos', None)
+      self.set('endPos', None)
+      self.set('middlePos', [])
+      self.selectOnePoint = False
+      self.selectTwoPoints = True
+      self.selectManyPoints = True
+      self.osdMenuState = OSD_EDIT
 
     elif message == "selectOnePoint":
       self.set('startPos', None)
       self.set('endPos', None)
+      self.set('middlePos', [])
       self.selectTwoPoints = True # we reuse the p2p menu
       self.selectOnePoint = True
+      self.selectManyPoints = False
+      self.osdMenuState = OSD_EDIT
 
-    elif message == "p2pRoute": # simple route, from here to selected point
+    elif message == "p2pRoute": # simple route, between two points
       toPos = self.get("endPos", None)
-      if toPos:
+      fromPos = self.get("startPos", None)
+      if toPos and fromPos:
         toLat, toLon = toPos
+        fromLat, fromLon = fromPos
+        middlePoints = self.get('middlePos', [])
+        print("Routing %f,%f to %f,%f through %d waypoints"
+              % (fromLat, fromLon, toLat, toLon, len(middlePoints)))
+        # TODO: wait message (would it be needed when using internet routing ?)
+        self.doRoute(fromLat, fromLon, toLat, toLon, waypoints=middlePoints)
+        self.set('needRedraw', True) # show the new route
 
-        fromPos = self.get("startPos", None)
-        if fromPos:
-          fromLat, fromLon = fromPos
-
-          print("Routing %f,%f to %f,%f" % (fromLat, fromLon, toLat, toLon))
-
-          # TODO: wait message (would it be needed when using internet routing ?)
-          self.doRoute(fromLat, fromLon, toLat, toLon)
-          self.set('needRedraw', True) # show the new route
+    elif message == "p2phmRoute": # simple route, from start to middle to end (handmade routing)
+      fromPos = self.get("startPos", None)
+      toPos = self.get("endPos", None)
+      if fromPos and toPos:
+        toLat, toLon = toPos
+        fromLat, fromLon = fromPos
+        middlePoints = self.get("middlePos", [])
+        print("Handmade route ")
+        print(fromLat, fromLon)
+        print(" through ")
+        print(middlePoints)
+        print(" to %f,%f" % (toLat, toLon))
+        war = way.fromHandmade(fromPos, middlePoints, toPos)
+        print("NO WAI")
+        print(war)
+        self.processAndSaveResults(war, "start", "end", time.time())
+        # start TbT navigation (if enabled)
+        self.startNavigation()
+        # switch to the route options OSD menu
+        self.osdMenuState = OSD_CURRENT_ROUTE
+        self.set('needRedraw', True) # show the new route
 
     elif message == "p2posRoute": # simple route, from here to selected point
       startPos = self.get('startPos', None)
       endPos = self.get('endPos', None)
       pos = self.get('pos', None)
       if pos is None: # well, we don't know where we are, so we don't know here to go :)
-        return
+        return None
 
       if startPos is None and endPos is None: # we know where we are, but we don't know where we should go :)
-        return
+        return None
 
       if startPos is not None: # we want a route from somewhere to our current position
         fromPos = startPos
@@ -190,13 +269,15 @@ class route(ranaModule):
       (toLat, toLon) = toPos
       (fromLat, fromLon) = fromPos
 
-      print("Routing %f,%f to %f,%f" % (fromLat, fromLon, toLat, toLon))
+      middlePoints = self.get("middlePos", [])
+      print("Routing %f,%f to %f,%f through %d waypoints"
+            % (fromLat, fromLon, toLat, toLon, len(middlePoints)))
 
-      self.doRoute(fromLat, fromLon, toLat, toLon)
+      self.doRoute(fromLat, fromLon, toLat, toLon, waypoints=middlePoints)
       self.set('needRedraw', True) # show the new route
 
     elif message == "route": # find a route
-      if type == 'md': # message-list based routing
+      if type == 'md': # message-list based unpack requires a string argument of length 4 routing
         if args:
           type = args['type']
           go = False
@@ -216,9 +297,10 @@ class route(ranaModule):
             try:
               self.doRoute(fromLat, fromLon, toLat, toLon)
             except Exception, e:
-              traceback.print_exc(file=sys.stdout)
               self.sendMessage('ml:notification:m:No route found;3')
               self.set('needRedraw', True)
+              print(e)
+              traceback.print_exc(file=sys.stdout)
             if "show" in args:
               """switch to map view and go to start/destination, if requested"""
               where = args['show']
@@ -231,17 +313,17 @@ class route(ranaModule):
             self.set('needRedraw', True) # show the new route
       else: # simple route, from here to selected point
         # disable the point selection GUIs
+        self.selectManyPoints = False # handmade
         self.selectTwoPoints = False
         self.selectOnePoint = False
+        self.osdMenuState = OSD_CURRENT_ROUTE
         toPos = self.get("selectedPos", None)
         if toPos:
           toLat, toLon = [float(a) for a in toPos.split(",")]
-
           fromPos = self.get("pos", None)
           if fromPos:
             (fromLat, fromLon) = fromPos
             print("Routing %f,%f to %f,%f" % (fromLat, fromLon, toLat, toLon))
-
             # TODO: wait message (would it be needed when using internet routing ?)
             self.doRoute(fromLat, fromLon, toLat, toLon)
             self.set('needRedraw', True) # show the new route
@@ -254,7 +336,6 @@ class route(ranaModule):
       if not self.directions:
         print("route: the route is empty, so it will not be stored")
         return
-
       loadTracklogs.storeRouteAndSetActive(self.directions.getPointsLLE(), '',
         'online') # TODO: rewrite this when we support more routing providers
 
@@ -269,6 +350,13 @@ class route(ranaModule):
       entryText = self.get('startAddress', "")
       entry.entryBox(self, 'start', 'Input the start address', entryText)
 
+    elif message == 'middleInput': # handmade
+      entry = self.m.get('textEntry', None)
+      if entry is None:
+        return
+      entryText = ""
+      entry.entryBox(self ,'middle','Input the directions for this middle point',entryText)
+
     elif message == 'destinationInput':
       entry = self.m.get('textEntry', None)
       if entry is None:
@@ -308,7 +396,7 @@ class route(ranaModule):
         self.set('destinationAddress', posString) # also store in the persistent dictionary
 
     elif message == 'reroute':
-      if type == 'ms' and args == "fromPosToDest":
+      if type == 'ms' and args == "fromPosToDest" and self.selectManyPoints == False: # handmade
         """reroute from current position to destination"""
         # is there a destination and valid position ?
         print("route: rerouting from current position to last destination")
@@ -330,8 +418,16 @@ class route(ranaModule):
         # redraw the screen to show the change
         self.set('needRedraw', True)
 
-  def doRoute(self, fromLat, fromLon, toLat, toLon):
+    elif type == "ms" and message == "setOSDState":
+      self.osdMenuState = int(args)
+      self.set('needRedraw', True) # show the new menu
+
+  def doRoute(self, fromLat, fromLon, toLat, toLon, waypoints=None):
     """Route from one point to another, and set that as the active route"""
+    if not waypoints: waypoints = []
+    # try to make sure waypoints are (lat, lon) tuples
+    waypoints = map(lambda x: (x[0],x[1]), waypoints)
+
     # clear old addresses
     self.startAddress = None
     self.destinationAddress = None
@@ -347,18 +443,26 @@ class route(ranaModule):
     if provider == "Monav":
       sentTimestamp = time.time()
       print('routing: using Monav as routing provider')
-      waypoints = [(fromLat, fromLon), (toLat, toLon)]
-      result = self.getMonavRoute(waypoints)
+      monavWaypoints = [(fromLat, fromLon), (toLat, toLon)] # start
+      # disable additional waypoint usage due to a protobuf serialization bug in monav-server,
+      # that prevents getting results from more than 2 waypoints
+#      monavWaypoints.extend(waypoints) # waypoints
+
+      result = self.getMonavRoute(monavWaypoints)
       #TODO: asynchronous processing & error notifications
       # as monav is VERY fast for routing, the routing might still get done
       # asynchronously, but the work-in-progress overlay might show up
       # only once the search takes longer than say 2 seconds
-      self._handleResults("MonavRoute", (result, waypoints[0], waypoints[-1], sentTimestamp))
+      self._handleResults("MonavRoute", (result, monavWaypoints[0], monavWaypoints[-1], sentTimestamp))
 
     else: # use Google Directions as fallback
       online = self.m.get('onlineServices', None)
+      # prepare waypoints for Google Directions as a list of strings
+      gdWaypoints = map(lambda x: "(%f, %f)" % (x[0],x[1]), waypoints)
+
       if online:
-        online.googleDirectionsLLAsync((fromLat, fromLon), (toLat, toLon), self._handleResults, "onlineRoute")
+        online.googleDirectionsLLAsync((fromLat, fromLon), (toLat, toLon),
+          self._handleResults, "onlineRoute", waypoints=gdWaypoints)
 
   def getMonavRoute(self, waypoints):
     mode = self.get('mode', 'car')
@@ -441,6 +545,7 @@ class route(ranaModule):
 
   def _handleResults(self, key, resultsTuple):
     """handle a routing result"""
+    routingSuccess = False
     if key in ("onlineRoute", "onlineRouteAddress2Address"):
       if key == "onlineRoute":
         (directions, start, destination, routeRequestSentTimestamp) = resultsTuple
@@ -451,18 +556,20 @@ class route(ranaModule):
 
         if directions: # is there actually something in the directions ?
           # create the directions Way object
-          self.duration = directions['Directions']['Duration']['html']
+          self.durationString = directions['routes'][0]['legs'][0]['duration']['text']
           dirs = way.fromGoogleDirectionsResult(directions)
           #TODO: use seconds from Way object directly
           #(needs seconds to human representation conversion)
           #self.duration = dirs.getDuration()
           self.processAndSaveResults(dirs, start, destination, routeRequestSentTimestamp)
+          routingSuccess = True
+          self.startNavigation()
       elif key == "onlineRouteAddress2Address":
         (directions, start, destination, routeRequestSentTimestamp) = resultsTuple
         # remove any possible prev. route description, so new a new one for this route is created
         self.text = None
         if directions: # is there actually something in the directions ?
-          self.duration = directions['Directions']['Duration']['html']
+          self.durationString = directions['routes'][0]['legs'][0]['duration']['text']
           #TODO: use seconds from Way object directly
           #(needs seconds to human representation conversion)
           #self.duration = dirs.getDuration()
@@ -470,17 +577,15 @@ class route(ranaModule):
           # create the directions Way object
           dirs = way.fromGoogleDirectionsResult(directions)
           self.processAndSaveResults(dirs, start, destination, routeRequestSentTimestamp)
-        # handle navigation autostart
-      autostart = self.get('autostartNavigationDefaultOnAutoselectTurn', 'enabled')
-      if autostart == 'enabled':
-        self.sendMessage('ms:turnByTurn:start:%s' % autostart)
+          routingSuccess = True
+          self.startNavigation()
       self.set('needRedraw', True)
     elif key == "MonavRoute":
       (result, start, destination, routeRequestSentTimestamp) = resultsTuple
       directions, returnCode = result
 
       if returnCode == ROUTING_SUCCESS:
-        self.duration = "" # TODO : correct predicted route duration
+        self.durationString = "" # TODO : correct predicted route duration
 
         # provided a turn detection function to the way object
         tbt = self.m.get("turnByTurn", None)
@@ -492,10 +597,9 @@ class route(ranaModule):
         dirs = way.fromMonavResult(directions, getTurns)
         self.processAndSaveResults(dirs, start, destination, routeRequestSentTimestamp)
 
+        routingSuccess = True
         # handle navigation autostart
-        autostart = self.get('autostartNavigationDefaultOnAutoselectTurn', 'enabled')
-        if autostart == 'enabled':
-          self.sendMessage('ms:turnByTurn:start:%s' % autostart)
+        self.startNavigation()
         self.set('needRedraw', True)
 
       else: # routing failed
@@ -517,16 +621,27 @@ class route(ranaModule):
       self.destinationAddress = resultsTuple
       self.text = None # clear route detail cache
 
-  def processAndSaveResults(self, directions, start, destination, routeRequestSentTimestamp):
+    # if routing was successful switch to the current-route OSD menu
+    if routingSuccess:
+      self.osdMenuState = OSD_CURRENT_ROUTE
+
+  def startNavigation(self):
+    """handle navigation autostart"""
+    autostart = self.get('autostartNavigationDefaultOnAutoselectTurn', 'enabled')
+    if autostart == 'enabled':
+      self.sendMessage('ms:turnByTurn:start:%s' % autostart)
+
+
+  def processAndSaveResults(self, directionsWay, start, destination, routeRequestSentTimestamp):
     """process and save routing results"""
     self.routeRequestSentTimestamp = routeRequestSentTimestamp
     proj = self.m.get('projection', None)
     if proj:
-      self.pxpyRoute = [proj.ll2pxpyRel(x[0], x[1]) for x in directions.getPointsLLE()]
-    self.processAndSaveDirections(directions)
+      self.pxpyRoute = [proj.ll2pxpyRel(x[0], x[1]) for x in directionsWay.getPointsLLE()]
+    self.processAndSaveDirections(directionsWay)
 
-    (fromLat, fromLon) = directions.getPointByID(0).getLL()
-    (toLat, toLon) = directions.getPointByID(-1).getLL()
+    (fromLat, fromLon) = directionsWay.getPointByID(0).getLL()
+    (toLat, toLon) = directionsWay.getPointByID(-1).getLL()
 
     """use coordinates for start dest or use first/last point from the route
        if start/dest coordinates are unknown (None)"""
@@ -568,30 +683,36 @@ class route(ranaModule):
 
     for step in steps:
       originalMessage = "".join(step.getMessage())
-      message = step.getMessage() #TODO: make a method for this
-      message = re.sub(r'<div[^>]*?>', '\n<i>', message)
-      message = re.sub(r'</div[^>]*?>', '</i>', message)
-      message = re.sub(r'<wbr/>', ', ', message)
-      message = re.sub(r'<wbr>', ', ', message)
-      step.setMessage(message)
-      # special processing of the original message for Espeak
-      message = originalMessage
+      try:
+        message = step.getMessage() #TODO: make a method for this
+        message = re.sub(r'<div[^>]*?>', '\n<i>', message)
+        message = re.sub(r'</div[^>]*?>', '</i>', message)
+        message = re.sub(r'<wbr/>', ', ', message)
+        message = re.sub(r'<wbr>', ', ', message)
+        step.setMessage(message)
+        # special processing of the original message for Espeak
+        message = originalMessage
 
-      # check if cyrillic -> russian voice is enabled
-      cyrillicVoice = self.get('voiceNavigationCyrillicVoice', 'ru')
-      if cyrillicVoice:
-        message = self.processCyrillicString(message, cyrillicVoice)
+        # check if cyrillic -> russian voice is enabled
+        cyrillicVoice = self.get('voiceNavigationCyrillicVoice', 'ru')
+        if cyrillicVoice:
+          message = self.processCyrillicString(message, cyrillicVoice)
 
-      message = re.sub(r'<div[^>]*?>', '<br>', message)
-      message = re.sub(r'</div[^>]*?>', '', message)
-      message = re.sub(r'<b>', '<emphasis level="strong">', message)
-      message = re.sub(r'</b>', '</emphasis>', message)
+        message = re.sub(r'<div[^>]*?>', '<br>', message)
+        message = re.sub(r'</div[^>]*?>', '', message)
+        message = re.sub(r'<b>', '<emphasis level="strong">', message)
+        message = re.sub(r'</b>', '</emphasis>', message)
+
+        # apply external rules from a CSV file
+        for (regex, replacement) in self.directionsFilterRules:
+          # replace strings according to the csv file
+          message = regex.sub(replacement, message, re.UNICODE)
+        step.setSSMLMessage(message)
+      except Exception, e:
+        print("route: error during direction filtering")
+        print(e)
+        step.setSSMLMessage(message)
 
-      # apply external rules from a CSV file
-      for (regex, replacement) in self.directionsFilterRules:
-        # replace strings according to the csv file
-        message = regex.sub(replacement, message, re.UNICODE)
-      step.setSSMLMessage(message)
 
     # replace old message points with new ones
     directions.clearMessagePoints()
@@ -635,7 +756,7 @@ class route(ranaModule):
         else: # no cyrillic string in progress
           # just store the current substring
           outputString = outputString + ' ' + substring
-      # cleanup
+          # cleanup
     if cyrillicStringTemp: #is there an "open" cyrillic string ?
       cyrillicStringTemp += '</p>' # close the string
       outputString += ' ' + cyrillicStringTemp
@@ -643,14 +764,20 @@ class route(ranaModule):
     return outputString
 
   def drawScreenOverlay(self, cr):
+    showMenu = False
     menus = self.m.get('menu', None)
     if menus:
-      if not menus.buttonsHidingOn(): # check if the buttons should not be hidden
-        if self.directions: # current route info button
-          self.drawCurrentRouteInfoButton(cr)
+      # check if the buttons should not be hidden
+      showMenu = not menus.buttonsHidingOn() and self.osdMenuState is not None
+
+    if showMenu :
+      if self.osdMenuState == OSD_EDIT:
+        self.drawRoutePlaningMenu(cr)
+      elif self.osdMenuState == OSD_CURRENT_ROUTE: # current route info button
+        self.drawCurrentRouteButton(cr)
+      elif self.osdMenuState == OSD_ROUTE_OPTIONS:
+        self.drawCurrentRouteOptionsMenu(cr)
 
-        if self.selectTwoPoints: # point selection menu
-          self.drawTwoPointsMenu(cr)
 
     #register clickable areas for manual point input
     if self.expectStart:
@@ -658,6 +785,11 @@ class route(ranaModule):
       (x, y, w, h) = self.get('viewport')
       if clickHandler is not None:
         clickHandler.registerXYWH(x, y, x + w, y + h, 'route:setStart')
+    if self.expectMiddle: # handmade
+      clickHandler = self.m.get('clickHandler', None)
+      (x,y,w,h) = self.get('viewport')
+      if clickHandler is not None:
+        clickHandler.registerXYWH(x, y, x+w, y+h, 'route:setMiddle')
     if self.expectEnd:
       clickHandler = self.m.get('clickHandler', None)
       (x, y, w, h) = self.get('viewport')
@@ -732,20 +864,20 @@ class route(ranaModule):
       # according to numerous sources, list comprehensions should be faster than for loops and map+lambda
       # if its faster in this case too has not been determined
 
-
-      """
-      routing result drawing algorithm
-      adapted from TangoGPS source (tracks.c)
-      works surprisingly good :)
-      """
+      # routing result drawing algorithm
+      # adapted from TangoGPS source (tracks.c)
+      # works surprisingly good
 
       z = proj.zoom
-
       # these setting seem to work the best for routing results:
       # (they have a different structure than logging traces,
       # eq. long segments delimited by only two points, etc)
       # basically, routing results have only the really needed points -> less points than traces
-      if 16 > z > 10:
+      if self.handmade or len(self.pxpyRoute) < 20:
+        # handmade routes usually have very few points and we don't want to skip any
+        # also handle very short routes that might have the same issue
+        modulo = 1
+      elif 16 > z > 10:
         modulo = 2 ** (14 - z)
       elif z <= 10:
         modulo = 16
@@ -792,26 +924,37 @@ class route(ranaModule):
     if self.selectTwoPoints:
       self.drawPointSelectors(cr)
 
-    #    print("Redraw took %1.9f ms" % (1000 * (clock() - start1)))
+      #    print("Redraw took %1.9f ms" % (1000 * (clock() - start1)))
 
   def getCurrentDirections(self):
-    # return the current route
+    """return the current route"""
     return self.directions, self.routeRequestSentTimestamp
 
-  def drawCurrentRouteInfoButton(self, cr):
+  def drawCurrentRouteButton(self, cr):
+    """draw the info button for the current route on the map screen"""
     (x, y, w, h) = self.get('viewport')
     menus = self.m.get('menu', None)
     dx = min(w, h) / 5.0
     dy = dx
     x1 = (x + w) - dx
     y1 = (y - dy) + h
-    if self.selectTwoPoints:
-      """move to avoid collision with the point selection menu"""
-      x1 -= dx
-      y1 -= dy
-    menus.drawButton(cr, x1, y1, dx, dy, 'info#route', "generic:;0.5;;0.5;;", 'set:menu:route#currentRouteBackToMap')
+    menus.drawButton(cr, x1, y1, dx, dy, 'tools#route', "generic:;0.5;;0.5;;", 'ms:route:setOSDState:%d' % OSD_ROUTE_OPTIONS)
 
-  def drawTwoPointsMenu(self, cr):
+  def drawCurrentRouteOptionsMenu(self, cr):
+    """draw the options for the current route on the map screen"""
+    (x, y, w, h) = self.get('viewport')
+    menus = self.m.get('menu', None)
+    dx = min(w, h) / 5.0
+    dy = dx
+    x1 = (x + w) - dx
+    y1 = (y - dy) + h
+    menus.drawButton(cr, x1-dx, y1, dx, dy,
+      'edit', "above:edit>generic:;0.5;;0.5;;", 'ms:route:setOSDState:%d' % OSD_EDIT)
+    menus.drawButton(cr, x1, y1, dx, dy,
+      'info', "above:info>generic:;0.5;;0.5;;", 'set:menu:route#currentRouteBackToMap|ms:route:setOSDState:%d' % OSD_EDIT)
+
+  def drawRoutePlaningMenu(self, cr):
+    """draw the onscreen menu for route planing"""
     (x, y, w, h) = self.get('viewport')
     dx = min(w, h) / 5.0
     dy = dx
@@ -820,17 +963,26 @@ class route(ranaModule):
     y1 = (y - dy) + h
 
     startIcon = "generic:;0.5;;0.5;;"
+    middleIcon = "generic:;0.5;;0.5;;"
     endIcon = "generic:;0.5;;0.5;;"
     if self.expectStart:
       startIcon = "generic:red;0.5;red;0.5;;"
+    if self.expectMiddle:
+      middleIcon = "generic:blue;0.5;blue;0.5;;"
     if self.expectEnd:
       endIcon = "generic:green;0.5;green;0.5;;"
 
     routingAction = 'route:p2pRoute'
     if self.selectOnePoint:
       routingAction = 'route:p2posRoute'
+    if self.handmade: # handmade
+      routingAction = 'route:p2phmRoute'
 
     menus.drawButton(cr, x1 - dx, y1, dx, dy, 'start', startIcon, "route:expectStart")
+    # Monav currently has a bug preventing waypoint routing
+    routingProvider = self.get('routingProvider', 'GoogleDirections')
+    if self.handmade or routingProvider == 'GoogleDirections':
+      menus.drawButton(cr, x1-dx, y1-dy, dx, dy, 'middle', middleIcon, "route:expectMiddle") # handmade
     menus.drawButton(cr, x1, y1 - dy, dx, dy, 'end', endIcon, "route:expectEnd")
     menus.drawButton(cr, x1, y1, dx, dy, 'route', "generic:;0.5;;0.5;;", routingAction)
 
@@ -842,6 +994,7 @@ class route(ranaModule):
     # draw point selectors
     proj = self.m.get('projection', None)
     fromPos = self.get('startPos', None)
+    middlePos = self.get('middlePos', None)
     toPos = self.get('endPos', None)
     if fromPos is not None:
       cr.set_line_width(10)
@@ -860,6 +1013,24 @@ class route(ranaModule):
       cr.stroke()
       cr.fill()
 
+    # only show middle point indicators in the
+    # edit menu mode at they might easily overlay the route otherwise
+    if middlePos and self.osdMenuState == OSD_EDIT:
+      for point in middlePos:
+        (lat,lon) = point[0], point[1]
+        cr.set_line_width(10)
+        cr.set_source_rgb(0, 0, 1)
+        (x, y) = proj.ll2xy(lat, lon)
+        cr.arc(x, y, 2, 0, 2.0 * math.pi)
+        cr.stroke()
+        cr.fill()
+
+        cr.set_line_width(8)
+        cr.set_source_rgba(0, 0, 1, 0.95) # transparent blue
+        cr.arc(x, y, 15, 0, 2.0 * math.pi)
+        cr.stroke()
+        cr.fill()
+
     if toPos is not None:
       cr.set_line_width(10)
       cr.set_source_rgb(0, 1, 0)
@@ -879,6 +1050,16 @@ class route(ranaModule):
     if key == 'start':
       self.startAddress = result
       self.set('startAddress', result)
+    elif key == 'middle':
+      mpList=self.get('middlePos',[])
+      if mpList:
+        # replace the last added point by the same point with
+        # message user wrote to the entry box
+        lastMiddlePoint = mpList.pop()
+        (lat, lon, elev, message) = lastMiddlePoint
+        message = result
+        mpList.append((lat, lon, elev, message))
+        self.set('middlePos', mpList)
     elif key == 'destination':
       self.destinationAddress = result
       self.set('destinationAddress', result)
@@ -918,8 +1099,8 @@ class route(ranaModule):
             self._geocodeStartAndDestination()
             self.routeDetailGeocodingTriggered = True
 
-        if self.duration:
-          duration = self.duration # a string describing the estimated time to finish the route
+        if self.durationString:
+          duration = self.durationString # a string describing the estimated time to finish the route
         else:
           duration = "? minutes"
         units = self.m.get('units', None) # get the correct units
@@ -979,6 +1160,9 @@ class route(ranaModule):
       menus.addItem('currentRouteTools', 'clear', 'generic', 'route:clear|set:menu:None')
 
     if menuName == "showAddressRoute":
+      self._drawAddressRoutingMenu(cr)
+
+  def _drawAddressRoutingMenu(self, cr):
       menus = self.m.get("menu", None)
       if menus:
         (e1, e2, e3, e4, alloc) = menus.threePlusOneMenuCoords()
@@ -1041,11 +1225,5 @@ class route(ranaModule):
     # stop the Monav server, if running
     if self.monav:
       self.monav.stopServer()
-
-if(__name__ == '__main__'):
-  d = {'transport': 'car'}
-  a = route({}, d)
-  a.doRoute(51.51565, 0.06036, 51.65299, -0.19974) # Beckton -> Barnet
-  print(a.route)
   
   
